package com.yihu.wlyy.service.template;

import com.yihu.wlyy.entity.template.DoctorGuidanceTemp;
import com.yihu.wlyy.repository.template.DoctorGuidanceTempDao;
import com.yihu.wlyy.service.BaseService;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.PageRequest;
import org.springframework.stereotype.Service;

import java.util.Date;
import java.util.List;

/**
 * 医生健康指导模板
 * <p>
 * Created by lyr-pc on 2017/2/3.
 */
@Service
public class DoctorGuidanceTempService extends BaseService {

    @Autowired
    DoctorGuidanceTempDao guidanceTempDao;

    /**
     * 添加指导模板
     *
     * @param doctor  医生
     * @param content 指导内容
     * @return
     */
    public DoctorGuidanceTemp add(String doctor, String content) {
        DoctorGuidanceTemp guidanceTemp = new DoctorGuidanceTemp();

        guidanceTemp.setCode(getCode());
        guidanceTemp.setOwner(doctor);
        guidanceTemp.setContent(content);
        guidanceTemp.setSendTimes(0);
        guidanceTemp.setCreateTime(new Date());

        return guidanceTempDao.save(guidanceTemp);
    }

    /**
     * 修改指导模板
     *
     * @param code    模板编码
     * @param content 指导内容
     * @return
     */
    public DoctorGuidanceTemp modify(String code, String content) throws Exception {
        DoctorGuidanceTemp guidanceTemp = guidanceTempDao.findByCode(code);

        if (guidanceTemp == null) {
            throw new Exception("模板不存在");
        }

        guidanceTemp.setContent(content);

        return guidanceTempDao.save(guidanceTemp);
    }

    /**
     * 删除指导模板
     *
     * @param code 模板编码
     * @return
     */
    public int delete(String code) throws Exception {
        String[] codes = code.split(",");

        if (codes != null && codes.length > 0) {
            for (String temp : codes) {
                guidanceTempDao.deleteByCode(temp);
            }
        }

        return 1;
    }

    /**
     * 查询指导模板
     *
     * @param doctor 医生
     * @param type   模板类型
     * @return
     */
    public List<DoctorGuidanceTemp> list(String doctor, String type) throws Exception {
        List<DoctorGuidanceTemp> temps = null;

        if (type.equals("1")) {
            temps = guidanceTempDao.findByOwner("system");
        } else if (type.equals("2")) {
            temps = guidanceTempDao.findByOwner(doctor);
        } else {
            temps = guidanceTempDao.findByOwnerAndSystem(doctor);
        }

        return temps;
    }


}
